/* graveman.h
 * Copyright (C) 2004, 2005 Sylvain Cresto <sylvain.cresto@tiscali.fr>
 *
 * This file is part of graveman!
 *
 * graveman! is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 * 
 * graveman! is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with program; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA. 
 * 
 * URL: http://scresto.site.voite.fr/
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include <dirent.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <math.h>
#include <glib/gprintf.h>
#include "config.h"
#include "error.h"
/* gi18b-lib.h doit etre inclue APRES config.h ! */
#include <glib/gi18n-lib.h>
#include <glade/glade.h>

#define _BOOLEAN_CONF(valeur) (valeur == TRUE ? "1" : "0")

/* convertire un texte en utf8 */
#define _UTF8(texte) g_locale_to_utf8(texte, -1, NULL, NULL, NULL)
/* recupere un nom de fichier de puis de l'utf8 */
#define _FILENAME(texte) g_filename_from_utf8(texte, -1, NULL, NULL, NULL)
/* message warning + affiche errno en utf8 */
#define _WARN(texte, ...) g_warning(texte, ## __VA_ARGS__); \
                          g_warning("%s\n", g_strerror(errno))

/* non null */
#define _NN(valeur) valeur ? valeur : ""

#define _hash() g_hash_table_new(g_str_hash, g_str_equal)
#define _hash_autodestroy() g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free)

#define _SWAP_COMBO(key, value) strcpy(key+strlen(key)-strlen("combo"), value)
                          
/* message de debug en utf8 */
#ifdef DEBUG
#define _DEB(texte, ...)  g_log(NULL, G_LOG_LEVEL_DEBUG, texte, ## __VA_ARGS__)
#else
#define _DEB(texte, ...) /* debug */
#endif

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  gtk_widget_set_name(GTK_WIDGET(widget), name); \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  gtk_widget_set_name(GTK_WIDGET(widget), name); \
  g_object_set_data (G_OBJECT (component), name, widget)

#define MAX_SPEED 54

#define GRAVEUR_ONLY 1
#define ALL 2
#define ISO 4

#define TYPE_CD    1
#define TYPE_DIR   2
#define TYPE_FILE  3
#define TYPE_MP3   4
#define TYPE_IMAGEBOOT 5

#define SEARCH_LECTEUR  1
#define SEARCH_PROG     2
#define WRITE_CONFIG    4
#define UPDATE_SIZE     8

/* gestion CD de boot */
#define CDBOOT_NONE     0
#define CDBOOT_IMAGE    1
 
typedef enum {
  PB_UPDATE_AUDIO,
  PB_UPDATE_DATA,
  PB_NONE
} Ttypemajpbar;
  
typedef enum {
  CONF_IGNORE,
  CONF_PROG,
  CONF_DIR,
  CONF_STRING,
  CONF_BOOLEAN
} Ttypeconf;

#ifdef ENABLE_MP3
  #define MP3 ".mp3"
#endif
#ifdef ENABLE_OGG
  #define OGG ".ogg"
#endif
#define WAV ".wav"

#define _BUF_SIZE 100
#ifndef MAXPATHLEN
  #define MAXPATHLEN 2048
#endif

#define INIT_GDRIVES(quel) Gldrives[quel].type = Gldrives[quel].vitesse = -1; \
                           *Gldrives[quel].dev = *Gldrives[quel].name = 0

typedef struct {
  Ttypeconf type;
  gchar *entry;
  gboolean indispensable;
  gchar *defaultvalue;
} TProgRequis;

#define _READ_CDR     1
#define _WRITE_CDR    2
#define _WRITE_CDRW   4
#define _WRITE_DUMMY  8
#define DRIVE_READER(value) (Gldrives[value].type & _READ_CDR)
#define DRIVE_WRITER(value) (Gldrives[value].type & _WRITE_CDR)
#define DRIVE_CDRW_WRITER(value) (Gldrives[value].type & _WRITE_CDRW)
#define DRIVE_ICONE(value) (Gldrives[value].type & _WRITE_CDR) ? GCDRWpetiteIcone : GCDpetiteIcone

typedef struct {
  char name[256];
  char dev[64];
  int  type;
  short vitesse;
} Tdrive_info;

typedef struct {
  gchar *basecode;
  gchar *label;
  gboolean defaut;
} Tmyentry;

typedef struct {
  gulong size;
  gchar *labeldata;
  gchar *labelaudio;
} Tmediaentry;

/* audio.c */
extern gboolean preparetrack(GHashTable *Ahash, GError **Aerror);
extern gchar *make_audio_fileslist(gint Anbrfiles);

/* data.c */
extern gboolean preparedata(GHashTable *Ahash, GError **Aerror);

/* interface.c */
extern GdkPixbuf *GDirIcone;
extern GdkPixbuf *GFileIcone;
extern GdkPixbuf *GFileImage;
extern GdkPixbuf *GpetiteGraver;
extern GdkPixbuf *GISOpetiteIcone;
extern GdkPixbuf *GGraver;
extern GdkPixbuf *GEraseCdrw;
extern GdkPixbuf *GCDIcone;
extern GdkPixbuf *GAudioIcone;
extern GdkPixbuf *GCDRWIcone;
extern GdkPixbuf *GCDpetiteIcone;
extern GdkPixbuf *GCDRWpetiteIcone;
extern GSList *Gmedialist;
extern Tmediaentry Gdefaultmedia[];
extern void remplirecombovitesse(GtkComboBox *Acombo, gchar *Alecident);
extern void show_graverwindow(GHashTable *Ahash, gchar *Aaction, gchar *Atitle);
extern GtkWidget* create_addfileselection (GtkWindow *AParent);
extern GtkWidget* create_adddirselection (GtkWindow *AParent);
extern GtkMenu *create_gtkpopupmenudata(GtkWidget *Aliste, GHashTable *Ahash);
extern GtkMenu *create_gtkpopupmenuaudio(GtkWidget *Aliste, GHashTable *Ahash);
extern gboolean gtk_do_eventbuttonpress(GtkWidget *Aliste, GdkEventButton *Aevent, gpointer Adata);
extern GtkWidget* create_addpisteselection (GtkWindow *AParent);
extern GladeXML *create_window_encours(GtkWidget *AParent);
extern GtkWidget* manage_main_window (guint32 *Aaudiolen, guint32 *Aaudiomaxlen,
        guint32 *Adatasize, guint32 *Adatamaxsize);
extern GtkWidget* create_fileselection1 (GtkWindow *AParent, GtkFileChooserAction AType);
extern void about_graveman (void);
extern GHashTable *create_dialog_proprietes (GtkWidget *Aparent);
extern void welcome_to_graveman(void);
extern void switch_pbar_status(GtkProgressBar *Apbar, guint32 Amaxlen, guint32 Acurlen, Ttypemajpbar Aupdatetext);
extern void prepare_combobox_graveur(GladeXML *Axml, GHashTable *Ahash, gchar *Abasename);
extern void prepare_combobox_vitesse(GtkComboBox *Acombobox);
extern void update_combo_graveur(GHashTable *Ahash);
extern void prepare_listedata(GtkTreeView *Atreedata);
extern void prepare_listeaudio(GtkTreeView *Atreeaudio);
extern void prepare_popups(GladeXML *Axml, GHashTable *Ahash);
extern void prepare_disable_when_iso(GHashTable *Ahash, gchar *Abasename, const gchar *Afirst, ...);
extern void prepare_disable_when_notcdrw(GHashTable *Ahash, gchar *Abasename, const gchar *Afirst, ...);
extern void show_erasecdrwwindow(GHashTable *Ahash);
extern void show_fixcdwindow(GHashTable *Ahash);
extern GHashTable *addmediaformat(GHashTable *Aparent);

/* log.c */
extern GQuark graveman_error_quark (void);

/* main.c */
extern GladeXML *Gxml;

/* config.c */
extern TProgRequis Glisteprogrequis[];
extern void config_init();
extern void clean_config();
extern gchar *get_config_file_name();
extern gint is_config_valide();
extern gint manage_new_config(GHashTable *Ahash, gshort Aop, GError **Aerror);
extern gint Gfifosize;
extern gchar *conf_get_string(gchar *Anom);
extern gchar *conf_get_string_def(gchar *Anom, gchar *Adefault);
extern gint conf_get_int(gchar *Anom);
extern gboolean conf_get_boolean(gchar *Anom);
extern void conf_store_value(gchar *Aentry, gchar *Avalue);
extern void conf_store_int(gchar *Aentry, gint Avalue);
extern gint write_conf(gchar *AFichier, GError **Aerror);
extern void maj_proprietes_prog(GHashTable *Ahash);
extern void remplirecombo (GtkComboBox *Acombo, gchar *Abasename, Tmyentry *Aentry, gboolean Adoinit);
extern void remplirecombograveur(GtkComboBox *Acombo, gint Aquoi);
extern void remplirelistegraveur(GtkTreeView *Aliste);
extern gboolean read_config(GError **Aerror);
extern void restore_main_window_pos(GtkWidget *Awindow1);

/* matos.c */
extern Tdrive_info Gldrives[128];
extern unsigned int Gnumdrives;
extern Tdrive_info *findlecteur(gchar *Aname);
extern gint get_maxspeed(int Anum);
extern gboolean burn_data(GHashTable *Ahash, GError **Aerror);
extern gboolean burn_audio(GHashTable *Ahash, GError **Aerror);
extern gboolean burn_copy(GHashTable *Ahash, GError **Aerror);
extern gboolean deltempdir(gchar *Ldirpath);
extern gboolean is_duplicate_drives(gchar *Abustype, gchar *Adev);

/* cdrecord.c */
extern gchar *burn_data_getextrapara(GHashTable *Ahash);
extern int scan_for_drives(GError **Aerror);
extern gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror);
extern gboolean gravepiste(GHashTable *Ahash, GError **Aerror);
extern gboolean maj_drive_info(gint Anumdrive, GError **Aerror);
extern gboolean perform_erase_cdrw(GHashTable *Ahash, GError **Aerror);
extern gboolean perform_fix_cd(GHashTable *Ahash, GError **Aerror);

/* mkisofs.c */
extern gchar *make_image_getextrapara(GHashTable *Ahash);
extern gboolean make_iso(GHashTable *Ahash, GError **Aerror);

/* mp3.c */
extern int getMp3Info(gchar *Apath, gchar **Atitle, gchar **Aalbum, gchar **Aartist, guint32 *Alength, GError **Aerror);

/* ogg.c */
extern int getOggInfo(gchar *Apath, gchar **Atitle, gchar **Aalbum, gchar **Aartist, guint32 *Alength, GError **Aerror);

/* tools.c */
extern char *ltrim(char *Abuf);
extern gchar *belleHeure(gchar *Abuf, gchar *Adst);
extern gint sc_strcountv(gchar **Aarray);
extern void sc_strstripv(gchar **Aarray);
extern gchar *get_combo_value(GtkWidget *Acombo);
extern gint make_int(gchar *Abuf);
extern gshort make_short(gchar *Abuf);
extern gchar *sc_realloc_cat(gchar *Asep, gchar *Asrc, gchar *Aplus);
extern gchar *format_length(guint32 Alennbr);
extern gchar *format_size(guint32 Asize);
extern gboolean sc_str_has_casesuffix(gchar *Astr, gchar *Asuffix);
extern gpointer sc_hash_table_lookupv(GHashTable *Ahash, gchar *Akeyformat, ...);
extern gchar *sc_get_filename(gchar *Afilename);
extern void sc_line_data_copy(GtkTreeModel *Amodel, GtkTreeIter *Aiterde, GtkTreeIter *Aitervers);
extern gboolean sc_str_match(gchar *Lpattern, gchar *Lstr);

/* readcd.c */
extern gint make_image(GHashTable *Ahash, GError **Aerror);
#if 0
extern gboolean copy_on_the_fly(GtkWidget *Awindow1, GtkWidget *Aparent, GError **Aerror);
#endif

/* sox.c */
extern gboolean SoundToCdr(GHashTable *Ahash, gchar *AFichier, gchar *AVers, GError **Aerror);

/* wav.c */
extern gboolean getWavInfo(gchar *Apath, guint32 *Alength, GError **Aerror);

/* support.c */
extern void add_pixmap_directory (const gchar *Adirectory);
extern GtkWidget* create_pixmap (const gchar *Afilename);
extern GdkPixbuf* create_pixbuf (const gchar *Afilename);
extern GtkWidget *glade_to_hash(GladeXML *Axml, GHashTable *Ahash, gchar *Akey);
extern GtkWidget *glade_to_hashv(GladeXML *Axml, GHashTable *Ahash, const gchar *Akeyformat, ...);
extern void glade_to_hash_list(GladeXML *Axml, GHashTable *Ahash, const gchar *Afirst, ...);
extern GtkWidget *sc_glade_xml_get_widgetv(GladeXML *Axml, const gchar *Akeyformat, ...);

/* callbacks.c */
extern gboolean encours_callback(gpointer Adata);
extern void on_about (GtkMenuItem *Amenuitem, gpointer Adata);
extern gboolean on_closeabout (GtkWidget *Awidget, GdkEventButton *Aevent, gpointer Adata);
extern void on_togglecopiecd_clicked (GtkButton *Abutton, gpointer Adata);
extern void on_togglecdaudio_clicked (GtkButton *Abutton, gpointer Adata);
extern void on_togglecddata_clicked (GtkButton *Abutton, gpointer Adata);
extern void on_selectgraveur(GtkWidget *Acombo, gpointer Adata);
extern void gtk_file_sel(GtkWidget *Abtn, gpointer Adata);
extern void confirm_erasecdrw (GtkWidget *Abtn, gpointer Adata);
extern void confirm_fixcd (GtkWidget *Abtn, gpointer Adata);
extern void confirm_burn (GtkWidget *Abtn, gpointer Adata);
extern void abort_gravure (GtkWidget *Abtn, gpointer Adata);
extern void gtk_data_addfile(GtkWidget *Abtn, gpointer Adata);
extern void gtk_data_adddirectory(GtkWidget *Abtn, gpointer Adata);
extern void gtk_data_removefiles(GtkWidget *Abtn, gpointer Adata);
extern void gtk_audio_removetrack(GtkWidget *Abtn, gpointer Adata);
extern gboolean gtk_do_eventlisteajoutfichiers(GtkWidget *Lliste, GdkEventButton *AEvent);
extern void gtk_data_renommerelement(GtkWidget *btn, gpointer Adata);
extern void gtk_data_creerrepertoire(GtkWidget *btn, gpointer Adata);
extern void gtk_audio_addtrack(GtkWidget *btn, gpointer Adata);
extern void gtk_data_expandelement(GtkWidget *btn, gpointer Adata);
extern void gonextpage(GtkWidget *Abtn, GtkNotebook *Anotebook);
extern void changemenu(GtkToggleButton *Abutton, gpointer Adata);
extern void text_buffer_callback(GtkTextBuffer *Aobj, gpointer Adata);
extern void menu_active_panel(GtkWidget *Amenuitem, gpointer Adata);
extern void gtk_combotext_changed(GtkWidget *Acombo, gpointer Adata);
extern void switchpage(GtkNotebook *Anotebook, GtkNotebookPage *Anotebookpage, guint Apagenum, gpointer Adata);
extern void menu_active_proprietes(GtkWidget *Amenuitem, gpointer Adata);
extern void cherchelecteur(GtkWidget *Aobj, gpointer data);
extern void chercheprog(GtkWidget *Aobj, gpointer data);
extern void abort_encours(GtkWidget *Abtn, gpointer Adata);
extern void menu_quit(GtkWidget *Aobj1, gpointer Adata);
extern void delete_event_quit(GtkWidget *Awidget, GdkEvent *Aevent, gpointer Adata);
extern void view_popupmenu_addaudio(GtkWidget *Aliste, GdkEventButton *Aevent, gpointer Adata);
extern void view_popupmenu_addfile(GtkWidget *Aliste, GdkEventButton *Aevent, gpointer Adata);
extern void change_remplissage(GtkWidget *Acombo, gpointer Adata);
extern gboolean on_select_files (GtkTreeSelection *Aselection, GtkTreeModel *Amodel,
                       GtkTreePath *Apath, gboolean Apath_currently_selected, gpointer Adata);
extern void audio_dnd_data_received (GtkWidget *Aliste, GdkDragContext *Acontext, gint Ax, gint Ay, GtkSelectionData *Aseldata, guint Ainfo, guint Atime, gpointer Adata);
extern void data_dnd_data_received (GtkWidget *Aliste, GdkDragContext *Acontext, gint Ax, gint Ay, GtkSelectionData *Aseldata, guint Ainfo, guint Atime, gpointer Adata);
extern void menu_start_new(GtkMenuItem *Amenuitem, gpointer Adata);
extern void on_addmediaformat(GtkWidget *Abtn, gpointer Adata);
extern void update_media_unit(GtkWidget *Acombo, gpointer Adata);
extern gboolean update_media_fields(GtkWidget *Awidget, gpointer Adata);
extern void gtk_def_imageboot(GtkWidget *Abtn, gpointer Adata);

/*
 * vim:et:ts=8:sts=2:sw=2
 */
